MmapFileWriterExt

Trait MmapFileWriterExt 

pub trait MmapFileWriterExt {
Show 26 methods // Required methods fn write_i8(&mut self, n: i8) -> Result<()>; fn write_i16(&mut self, n: i16) -> Result<()>; fn write_i16_le(&mut self, n: i16) -> Result<()>; fn write_i32(&mut self, n: i32) -> Result<()>; fn write_i32_le(&mut self, n: i32) -> Result<()>; fn write_i64(&mut self, n: i64) -> Result<()>; fn write_i64_le(&mut self, n: i64) -> Result<()>; fn write_isize(&mut self, n: isize) -> Result<()>; fn write_isize_le(&mut self, n: isize) -> Result<()>; fn write_i128(&mut self, n: i128) -> Result<()>; fn write_i128_le(&mut self, n: i128) -> Result<()>; fn write_u8(&mut self, n: u8) -> Result<()>; fn write_u16(&mut self, n: u16) -> Result<()>; fn write_u16_le(&mut self, n: u16) -> Result<()>; fn write_u32(&mut self, n: u32) -> Result<()>; fn write_u32_le(&mut self, n: u32) -> Result<()>; fn write_u64(&mut self, n: u64) -> Result<()>; fn write_u64_le(&mut self, n: u64) -> Result<()>; fn write_usize(&mut self, n: usize) -> Result<()>; fn write_usize_le(&mut self, n: usize) -> Result<()>; fn write_u128(&mut self, n: u128) -> Result<()>; fn write_u128_le(&mut self, n: u128) -> Result<()>; fn write_f32(&mut self, n: f32) -> Result<()>; fn write_f32_le(&mut self, n: f32) -> Result<()>; fn write_f64(&mut self, n: f64) -> Result<()>; fn write_f64_le(&mut self, n: f64) -> Result<()>;
}
Available on crate feature sync only.
Extends MmapFileWriter with methods for writing numbers.

Required Methods

fn write_i8(&mut self, n: i8) -> Result<()>

Writes a signed 8 bit integer to the underlying writer. Note that since this writes a single byte, no byte order conversions are used. It is included for completeness.

fn write_i16(&mut self, n: i16) -> Result<()>

Writes a signed 16 bit integer(big endian) to the underlying writer.

fn write_i16_le(&mut self, n: i16) -> Result<()>

Writes a signed 16 bit integer(little endian) to the underlying writer.

fn write_i32(&mut self, n: i32) -> Result<()>

Writes a signed 32 bit integer(big endian) to the underlying writer.

fn write_i32_le(&mut self, n: i32) -> Result<()>

Writes a signed 32 bit integer(little endian) to the underlying writer.

fn write_i64(&mut self, n: i64) -> Result<()>

Writes a signed 64 bit integer(big endian) to the underlying writer.

fn write_i64_le(&mut self, n: i64) -> Result<()>

Writes a signed 64 bit integer(little endian) to the underlying writer.

fn write_isize(&mut self, n: isize) -> Result<()>

Writes a signed integer(big endian) to the underlying writer.

fn write_isize_le(&mut self, n: isize) -> Result<()>

Writes a signed integer(little endian) to the underlying writer.

fn write_i128(&mut self, n: i128) -> Result<()>

Writes a signed 128 bit integer(big endian) to the underlying writer.

fn write_i128_le(&mut self, n: i128) -> Result<()>

Writes a signed 128 bit integer(little endian) to the underlying writer.

fn write_u8(&mut self, n: u8) -> Result<()>

Writes an unsigned 8 bit integer to the underlying writer. Note that since this writes a single byte, no byte order conversions are used. It is included for completeness.

fn write_u16(&mut self, n: u16) -> Result<()>

Writes an unsigned 16 bit integer(big endian) to the underlying writer.

fn write_u16_le(&mut self, n: u16) -> Result<()>

Writes an unsigned 16 bit integer(little endian) to the underlying writer.

fn write_u32(&mut self, n: u32) -> Result<()>

Writes an unsigned 32 bit integer(big endian) to the underlying writer.

fn write_u32_le(&mut self, n: u32) -> Result<()>

Writes an unsigned 32 bit integer(little endian) to the underlying writer.

fn write_u64(&mut self, n: u64) -> Result<()>

Writes an unsigned 64 bit integer(big endian) to the underlying writer.

fn write_u64_le(&mut self, n: u64) -> Result<()>

Writes an unsigned 64 bit integer(little endian) to the underlying writer.

fn write_usize(&mut self, n: usize) -> Result<()>

Writes an unsigned integer(big endian) to the underlying writer.

fn write_usize_le(&mut self, n: usize) -> Result<()>

Writes an unsigned integer(little endian) to the underlying writer.

fn write_u128(&mut self, n: u128) -> Result<()>

Writes an unsigned 128 bit integer(big endian) to the underlying writer.

fn write_u128_le(&mut self, n: u128) -> Result<()>

Writes an unsigned 128 bit integer(little endian) to the underlying writer.

fn write_f32(&mut self, n: f32) -> Result<()>

Writes a IEEE754 single-precision (4 bytes, big endian) floating point number to the underlying writer.

fn write_f32_le(&mut self, n: f32) -> Result<()>

Writes a IEEE754 single-precision (4 bytes, little endian) floating point number to the underlying writer.

fn write_f64(&mut self, n: f64) -> Result<()>

Writes a IEEE754 single-precision (8 bytes, big endian) floating point number to the underlying writer

fn write_f64_le(&mut self, n: f64) -> Result<()>

Writes a IEEE754 single-precision (8 bytes, little endian) floating point number to the underlying writer

Implementors