pub struct AsyncMmapFileMut { /* private fields */ }tokio only.Expand description
A writable memory map file.
There is 3 status of this struct:
- Disk: mmap to a real file
- Memory: use
BytesMutto mock a mmap, which is useful for test and in-memory storage engine - Empty: a state represents null mmap, which is helpful for drop, remove, close the
AsyncMmapFileMut. This state cannot be constructed directly.
Implementations
Sourceimpl AsyncMmapFileMut
impl AsyncMmapFileMut
Sourcepub async fn create<P: AsRef<Path>>(path: P) -> Result<Self>
pub async fn create<P: AsRef<Path>>(path: P) -> Result<Self>
Create a new file and mmap this file
Notes
The new file is zero size, so, before write, you should truncate first.
Or you can use create_with_options and set max_size field for AsyncOptions to enable directly write
without truncating.
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileMutExt};
let mut file = AsyncMmapFileMut::create("tokio_async_create_test.txt").await.unwrap();
file.truncate(12).await;
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();Sourcepub async fn create_with_options<P: AsRef<Path>>(
path: P,
opts: AsyncOptions,
) -> Result<Self>
pub async fn create_with_options<P: AsRef<Path>>( path: P, opts: AsyncOptions, ) -> Result<Self>
Create a new file and mmap this file with AsyncOptions
Example
use fmmap::tokio::{AsyncOptions, AsyncMmapFileMut, AsyncMmapFileMutExt};
let opts = AsyncOptions::new()
// truncate to 100
.max_size(100);
let mut file = AsyncMmapFileMut::create_with_options("tokio_async_create_with_options_test.txt", opts).await.unwrap();
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();Sourcepub async fn open<P: AsRef<Path>>(path: P) -> Result<Self>
pub async fn open<P: AsRef<Path>>(path: P) -> Result<Self>
Open or Create(if not exists) a file and mmap this file.
Notes
If the file does not exist, then the new file will be open in zero size, so before do write, you should truncate first.
Or you can use open_with_options and set max_size field for AsyncOptions to enable directly write
without truncating.
Examples
File already exists
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt};
// mmap the file
let mut file = AsyncMmapFileMut::open("tokio_async_open_test.txt").await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.truncate("some modified data...".len() as u64).await.unwrap();
file.write_all("some modified data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// reopen to check content
let mut buf = vec![0; "some modified data...".len()];
let mut file = AsyncMmapFileMut::open("tokio_async_open_test.txt").await.unwrap();
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some modified data...".as_bytes());File does not exists
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt};
// mmap the file
let mut file = AsyncMmapFileMut::open("tokio_async_open_test.txt").await.unwrap();
file.truncate(100).await.unwrap();
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.truncate("some modified data...".len() as u64).await.unwrap();
file.write_all("some modified data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// reopen to check content
let mut buf = vec![0; "some modified data...".len()];
let mut file = AsyncMmapFileMut::open("tokio_async_open_test.txt").await.unwrap();
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some modified data...".as_bytes());Sourcepub async fn open_with_options<P: AsRef<Path>>(
path: P,
opts: AsyncOptions,
) -> Result<Self>
pub async fn open_with_options<P: AsRef<Path>>( path: P, opts: AsyncOptions, ) -> Result<Self>
Open or Create(if not exists) a file and mmap this file with AsyncOptions.
Examples
File already exists
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt, AsyncOptions};
// mmap the file
let opts = AsyncOptions::new()
// allow read
.read(true)
// allow write
.write(true)
// allow append
.append(true)
// truncate to 100
.max_size(100)
// mmap content after the sanity text
.offset("sanity text".as_bytes().len() as u64);
let mut file = AsyncMmapFileMut::open_with_options("tokio_async_open_with_options_test.txt", opts).await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.truncate(("some modified data...".len() + "sanity text".len()) as u64).await.unwrap();
file.write_all("some modified data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// reopen to check content
let mut buf = vec![0; "some modified data...".len()];
let mut file = AsyncMmapFileMut::open("tokio_async_open_with_options_test.txt").await.unwrap();
// skip the sanity text
file.read_exact(buf.as_mut_slice(), "sanity text".as_bytes().len()).unwrap();
assert_eq!(buf.as_slice(), "some modified data...".as_bytes());File does not exists
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt, AsyncOptions};
// mmap the file with options
let opts = AsyncOptions::new()
// allow read
.read(true)
// allow write
.write(true)
// allow append
.append(true)
// truncate to 100
.max_size(100);
let mut file = AsyncMmapFileMut::open_with_options("tokio_async_open_with_options_test.txt", opts).await.unwrap();
file.write_all("some data...".as_bytes(), 0).unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.truncate("some modified data...".len() as u64).await.unwrap();
file.write_all("some modified data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// reopen to check content
let mut buf = vec![0; "some modified data...".len()];
let mut file = AsyncMmapFileMut::open("tokio_async_open_with_options_test.txt").await.unwrap();
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some modified data...".as_bytes());Sourcepub async fn open_exist<P: AsRef<Path>>(path: P) -> Result<Self>
pub async fn open_exist<P: AsRef<Path>>(path: P) -> Result<Self>
Open an existing file and mmap this file
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt};
// create a temp file
let mut file = AsyncMmapFileMut::create("tokio_async_open_existing_test.txt").await.unwrap();
file.truncate(12).await.unwrap();
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// mmap the file
let mut file = AsyncMmapFileMut::open_exist("tokio_async_open_existing_test.txt").await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.truncate("some modified data...".len() as u64).await.unwrap();
file.write_all("some modified data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// reopen to check content
let mut buf = vec![0; "some modified data...".len()];
let mut file = AsyncMmapFileMut::open("tokio_async_open_existing_test.txt").await.unwrap();
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some modified data...".as_bytes());Sourcepub async fn open_exist_with_options<P: AsRef<Path>>(
path: P,
opts: AsyncOptions,
) -> Result<Self>
pub async fn open_exist_with_options<P: AsRef<Path>>( path: P, opts: AsyncOptions, ) -> Result<Self>
Open an existing file and mmap this file with AsyncOptions
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt, AsyncOptions};
// create a temp file
let mut file = AsyncMmapFileMut::create("tokio_async_open_existing_test_with_options.txt").await.unwrap();
file.truncate(23).await.unwrap();
file.write_all("sanity text".as_bytes(), 0).unwrap();
file.write_all("some data...".as_bytes(), "sanity text".as_bytes().len()).unwrap();
file.flush().unwrap();
drop(file);
// mmap the file
let opts = AsyncOptions::new()
// truncate to 100
.max_size(100)
// mmap content after the sanity text
.offset("sanity text".as_bytes().len() as u64);
let mut file = AsyncMmapFileMut::open_exist_with_options("tokio_async_open_existing_test_with_options.txt", opts).await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.truncate(("some modified data...".len() + "sanity text".len()) as u64).await.unwrap();
file.write_all("some modified data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
// reopen to check content, cow will not change the content.
let mut file = AsyncMmapFileMut::open("tokio_async_open_existing_test_with_options.txt").await.unwrap();
let mut buf = vec![0; "some modified data...".len()];
// skip the sanity text
// file.read_exact(buf.as_mut_slice(), "sanity text".as_bytes().len()).unwrap();
// assert_eq!(buf.as_slice(), "some modified data...".as_bytes());Sourcepub async fn open_cow<P: AsRef<Path>>(path: P) -> Result<Self>
pub async fn open_cow<P: AsRef<Path>>(path: P) -> Result<Self>
Open and mmap an existing file in copy-on-write mode(copy-on-write memory map backed by a file). Data written to the memory map will not be visible by other processes, and will not be carried through to the underlying file.
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt};
// create a temp file
let mut file = AsyncMmapFileMut::create("tokio_async_open_cow_test.txt").await.unwrap();
file.truncate(12).await.unwrap();
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
drop(file);
// mmap the file
let mut file = AsyncMmapFileMut::open_cow("tokio_async_open_cow_test.txt").await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.write_all("some data!!!".as_bytes(), 0).unwrap();
file.flush().unwrap();
// cow, change will only be seen in current caller
assert_eq!(file.as_slice(), "some data!!!".as_bytes());
drop(file);
// reopen to check content, cow will not change the content.
let mut file = AsyncMmapFileMut::open("tokio_async_open_cow_test.txt").await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());Sourcepub async fn open_cow_with_options<P: AsRef<Path>>(
path: P,
opts: AsyncOptions,
) -> Result<Self>
pub async fn open_cow_with_options<P: AsRef<Path>>( path: P, opts: AsyncOptions, ) -> Result<Self>
Open and mmap an existing file in copy-on-write mode(copy-on-write memory map backed by a file) with AsyncOptions.
Data written to the memory map will not be visible by other processes, and will not be carried through to the underlying file.
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileExt, AsyncMmapFileMutExt, AsyncOptions};
use std::io::SeekFrom;
// create a temp file
let mut file = AsyncMmapFileMut::create("tokio_async_open_cow_with_options_test.txt").await.unwrap();
file.truncate(23).await.unwrap();
file.write_all("sanity text".as_bytes(), 0).unwrap();
file.write_all("some data...".as_bytes(), "sanity text".as_bytes().len()).unwrap();
file.flush().unwrap();
drop(file);
// mmap the file
let opts = AsyncOptions::new()
// mmap content after the sanity text
.offset("sanity text".as_bytes().len() as u64);
let mut file = AsyncMmapFileMut::open_cow_with_options("tokio_async_open_cow_with_options_test.txt", opts).await.unwrap();
let mut buf = vec![0; "some data...".len()];
file.read_exact(buf.as_mut_slice(), 0).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());
// modify the file data
file.write_all("some data!!!".as_bytes(), 0).unwrap();
file.flush().unwrap();
// cow, change will only be seen in current caller
assert_eq!(file.as_slice(), "some data!!!".as_bytes());
drop(file);
// reopen to check content, cow will not change the content.
let mut file = AsyncMmapFileMut::open("tokio_async_open_cow_with_options_test.txt").await.unwrap();
let mut buf = vec![0; "some data...".len()];
// skip the sanity text
file.read_exact(buf.as_mut_slice(), "sanity text".as_bytes().len()).unwrap();
assert_eq!(buf.as_slice(), "some data...".as_bytes());Sourcepub fn freeze(self) -> Result<AsyncMmapFile>
pub fn freeze(self) -> Result<AsyncMmapFile>
Make the mmap file read-only.
Notes
If remove_on_drop is set to true, then the underlying file will not be removed on drop if this function is invoked. Read more
Returns an immutable version of this memory mapped buffer. If the memory map is file-backed, the file must have been opened with read permissions.
Errors
This method returns an error when the underlying system call fails, which can happen for a variety of reasons, such as when the file has not been opened with read permissions.
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileMutExt};
let mut file = AsyncMmapFileMut::create("tokio_async_freeze_test.txt").await.unwrap();
file.truncate(12).await;
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
// freeze
file.freeze().unwrap();Sourcepub fn freeze_exec(self) -> Result<AsyncMmapFile>
pub fn freeze_exec(self) -> Result<AsyncMmapFile>
Transition the memory map to be readable and executable. If the memory map is file-backed, the file must have been opened with execute permissions.
Notes
If remove_on_drop is set to true, then the underlying file will not be removed on drop if this function is invoked. Read more
Errors
This method returns an error when the underlying system call fails, which can happen for a variety of reasons, such as when the file has not been opened with execute permissions
Examples
use fmmap::tokio::{AsyncMmapFileMut, AsyncMmapFileMutExt};
let mut file = AsyncMmapFileMut::create("tokio_async_freeze_exec_test.txt").await.unwrap();
file.truncate(12).await;
file.write_all("some data...".as_bytes(), 0).unwrap();
file.flush().unwrap();
// freeze_exec
file.freeze_exec().unwrap();Sourcepub fn get_remove_on_drop(&self) -> bool
pub fn get_remove_on_drop(&self) -> bool
Returns whether remove the underlying file on drop.
Sourcepub fn set_remove_on_drop(&mut self, val: bool)
pub fn set_remove_on_drop(&mut self, val: bool)
Whether remove the underlying file on drop. Default is false.
Notes
If invoke AsyncMmapFileMut::freeze, then the file will
not be removed even though the field remove_on_drop is true.
Sourceimpl AsyncMmapFileMut
impl AsyncMmapFileMut
Sourcepub fn memory_with_capacity<P: AsRef<Path>>(path: P, cap: usize) -> Self
pub fn memory_with_capacity<P: AsRef<Path>>(path: P, cap: usize) -> Self
Sourcepub fn memory_from_string<P: AsRef<Path>>(path: P, src: String) -> Self
pub fn memory_from_string<P: AsRef<Path>>(path: P, src: String) -> Self
Sourcepub fn memory_from_str<P: AsRef<Path>>(path: P, src: &'static str) -> Self
pub fn memory_from_str<P: AsRef<Path>>(path: P, src: &'static str) -> Self